@@ -1,9 +1,13 @@
 lrEMplus <- function(X, dl = NULL, rob = FALSE, ini.cov = c("complete.obs", "multRepl"), frac = 0.65,
                      tolerance = 0.0001, max.iter = 50,
-                     rlm.maxit=150, suppress.print = FALSE, closure=NULL, z.warning=0.8, delta=NULL){
+                     rlm.maxit=150, suppress.print = FALSE, closure=NULL, z.warning=0.8, z.delete=TRUE, delta=NULL){
   
   if (any(X<0, na.rm=T)) stop("X contains negative values")
-  if (is.character(dl) || is.null(dl)) stop("dl must be a numeric vector or matrix")
+  if (is.character(dl)) stop("dl must be a numeric vector or matrix")
+  if (is.null(dl)){ # If dl not given use min per column
+      dl <- apply(X,2, function(x) min(x[x!=0]))
+      warning("No dl vector or matrix provided. The minimum observed values for each column used as detection limits.")
+    }
   if (is.vector(dl)) dl <- matrix(dl,nrow=1)
   dl <- as.matrix(dl) # Avoids problems when dl might be multiple classes
   if ((is.vector(X)) | (nrow(X)==1)) stop("X must be a data matrix")
@@ -32,19 +36,32 @@ lrEMplus <- function(X, dl = NULL, rob = FALSE, ini.cov = c("complete.obs", "mul
   X <- as.data.frame(apply(X,2,as.numeric),stringsAsFactors=TRUE)
   c <- apply(X,1,sum,na.rm=TRUE)
   
-  # Number of zeros or missing per column for warning
-  checkNumZerosCol <- apply(X,2,function(x) sum(is.na(x) | (x==0)))
+  checkNumZerosCol <- apply(X,2,function(x) sum(is.na(x)))
   if (any(checkNumZerosCol/nrow(X) >= z.warning)) {
     cases <- which(checkNumZerosCol/nrow(X) >= z.warning)
-    X <- X[,-cases]
-    warning(paste("Column ",cases," containing more than ",z.warning*100,"% zeros/unobserved values was deleted (pre-check out using function zPatterns/modify threshold using argument z.warning).\n",sep=""))
+    if (z.delete == TRUE){
+      X <- X[,-cases]
+      action <- "deleted"
+      warning(paste("Column no. ",cases," containing >",z.warning*100,"% zeros/unobserved values ",action," (can modify threshold using argument z.warning).\n",sep=""))
+    }
+    else{
+      action <- "found"
+      stop(paste("Column no. ",cases," containing >",z.warning*100,"% zeros/unobserved values ",action," (can modify threshold using argument z.warning. Check out with zPatterns()).\n",sep=""))
+    }
   }
   
-  checkNumZerosRow <- apply(X,1,function(x) sum(is.na(x) | (x==0)))
+  checkNumZerosRow <- apply(X,1,function(x) sum(is.na(x)))
   if (any(checkNumZerosRow/ncol(X) >= z.warning)) {
     cases <- which(checkNumZerosRow/ncol(X) >= z.warning)
-    X <- X[-cases,]
-    warning(paste("Row ",cases," containing more than ",z.warning*100,"% zeros/unobserved values was deleted (pre-check out using function zPatterns/modify threshold using argument z.warning).\n",sep=""))
+    if (z.delete == TRUE){
+      X <- X[,-cases]
+      action <- "deleted"
+      warning(paste("Column no. ",cases," containing >",z.warning*100,"% zeros/unobserved values ",action," (can modify threshold using argument z.warning).\n",sep=""))
+    }
+    else{
+      action <- "found"
+      stop(paste("Column no. ",cases," containing >",z.warning*100,"% zeros/unobserved values ",action," (can modify threshold using argument z.warning. Check out with zPatterns()).\n",sep=""))
+    }
   }
 
   if (nrow(dl)==1) dl <- matrix(rep(1,nn),ncol=1)%*%dl
