@@ -1,11 +1,8 @@
 multKM <-
-  function (X,label=NULL,dl=NULL,n.draws=1000,n.knots=NULL,z.warning=0.8)
+  function (X,label=NULL,dl=NULL,n.draws=1000,n.knots=NULL,z.warning=0.8,z.delete=TRUE)
   {
     
     if (any(X<0, na.rm=T)) stop("X contains negative values")
-    if (is.character(dl) || is.null(dl)) stop("dl must be a numeric vector or matrix")
-    if (is.vector(dl)) dl <- matrix(dl,nrow=1)
-    dl <- as.matrix(dl) # Avoids problems when dl might be multiple classes
     if ((is.vector(X)) | (nrow(X)==1)) stop("X must be a data matrix")
     if (is.null(label)) stop("A value for label must be given")
     if (!is.na(label)){
@@ -17,6 +14,13 @@ multKM <-
       if (any(X==0,na.rm=T)) stop("Zero values not labelled as censored values were found in the data set")
       if (!any(is.na(X),na.rm=T)) stop(paste("Label",label,"was not found in the data set"))
     }
+    if (is.character(dl)) stop("dl must be a numeric vector or matrix")
+    if (is.null(dl)){ # If dl not given use min per column
+      dl <- apply(X,2, function(x) min(x[x!=label]))
+      warning("No dl vector or matrix provided. The minimum observed values for each column used as detection limits.")
+    }
+    if (is.vector(dl)) dl <- matrix(dl,nrow=1)
+    dl <- as.matrix(dl) # Avoids problems when dl might be multiple classes
     if (ncol(dl)!=ncol(X)) stop("The number of columns in X and dl do not agree")
     if ((nrow(dl)>1) & (nrow(dl)!=nrow(X))) stop("The number of rows in X and dl do not agree")
     
@@ -53,15 +57,29 @@ multKM <-
     checkNumZerosCol <- apply(X,2,function(x) sum(is.na(x)))
     if (any(checkNumZerosCol/nrow(X) >= z.warning)) {
       cases <- which(checkNumZerosCol/nrow(X) >= z.warning)
-      X <- X[,-cases]
-      warning(paste("Column ",cases," containing more than ",z.warning*100,"% zeros/unobserved values was deleted (pre-check out using function zPatterns/modify threshold using argument z.warning).\n",sep=""))
+      if (z.delete == TRUE){
+        X <- X[,-cases]
+        action <- "deleted"
+        warning(paste("Column no. ",cases," containing >",z.warning*100,"% zeros/unobserved values ",action," (can modify threshold using argument z.warning).\n",sep=""))
+      }
+      else{
+        action <- "found"
+        stop(paste("Column no. ",cases," containing >",z.warning*100,"% zeros/unobserved values ",action," (can modify threshold using argument z.warning. Check out with zPatterns()).\n",sep=""))
+      }
     }
     
     checkNumZerosRow <- apply(X,1,function(x) sum(is.na(x)))
     if (any(checkNumZerosRow/ncol(X) >= z.warning)) {
       cases <- which(checkNumZerosRow/ncol(X) >= z.warning)
-      X <- X[-cases,]
-      warning(paste("Row ",cases," containing more than ",z.warning*100,"% zeros/unobserved values was deleted (pre-check out using function zPatterns/modify threshold using argument z.warning).\n",sep=""))
+      if (z.delete == TRUE){
+        X <- X[,-cases]
+        action <- "deleted"
+        warning(paste("Column no. ",cases," containing >",z.warning*100,"% zeros/unobserved values ",action," (can modify threshold using argument z.warning).\n",sep=""))
+      }
+      else{
+        action <- "found"
+        stop(paste("Column no. ",cases," containing >",z.warning*100,"% zeros/unobserved values ",action," (can modify threshold using argument z.warning. Check out with zPatterns()).\n",sep=""))
+      }
     }
     
     nn <- nrow(X); p <- ncol(X)
