@@ -1,13 +1,8 @@
 multRepl <-
-  function(X,label=NULL,dl=NULL,frac=0.65,imp.missing=FALSE,closure=NULL,z.warning=0.8,delta=NULL){
+  function(X,label=NULL,dl=NULL,frac=0.65,imp.missing=FALSE,closure=NULL,z.warning=0.8,z.delete=TRUE,delta=NULL){
     
     if (any(X<0, na.rm=T)) stop("X contains negative values")
-    if (imp.missing==FALSE){
-      if (is.character(dl) || is.null(dl)) stop("dl must be a numeric vector or matrix")
-      if (is.vector(dl)) dl <- matrix(dl,nrow=1)
-      dl <- as.matrix(dl) # Avoids problems when dl might be multiple classes
-    }
-    
+
     if (is.character(X)) stop("X is not a valid data matrix or vector.")
     if (is.null(label)) stop("A value for label must be given")
     if (!is.na(label)){
@@ -19,6 +14,15 @@ multRepl <-
       if (any(X==0,na.rm=T)) stop("Zero values not labelled as censored or missing values were found in the data set")
       if (!any(is.na(X),na.rm=T)) stop(paste("Label",label,"was not found in the data set"))
     }
+    if (imp.missing==FALSE){
+      if (is.character(dl)) stop("dl must be a numeric vector or matrix")
+      if (is.null(dl)){ # If dl not given use min per column
+        dl <- apply(X,2, function(x) min(x[x!=label]))
+        warning("No dl vector or matrix provided. The minimum observed values for each column used as detection limits.")
+      }
+      if (is.vector(dl)) dl <- matrix(dl,nrow=1)
+      dl <- as.matrix(dl) # Avoids problems when dl might be multiple classes
+    }
     if (is.vector(X)){
       if (imp.missing==TRUE) stop("Data matrix required: missing values cannot be imputed in single vectors")
       if (ncol(dl)!=ncol(as.data.frame(matrix(X,ncol=length(X)),stringsAsFactors=TRUE))) stop("The number of columns in X and dl do not agree")
@@ -51,15 +55,29 @@ multRepl <-
       checkNumZerosCol <- apply(X,2,function(x) sum(is.na(x)))
       if (any(checkNumZerosCol/nrow(X) >= z.warning)) {
         cases <- which(checkNumZerosCol/nrow(X) >= z.warning)
-        X <- X[,-cases]
-        warning(paste("Column ",cases," containing more than ",z.warning*100,"% zeros/unobserved values was deleted (pre-check out using function zPatterns/modify threshold using argument z.warning).\n",sep=""))
+        if (z.delete == TRUE){
+          X <- X[,-cases]
+          action <- "deleted"
+          warning(paste("Column no. ",cases," containing >",z.warning*100,"% zeros/unobserved values ",action," (can modify threshold using argument z.warning).\n",sep=""))
+        }
+        else{
+          action <- "found"
+          stop(paste("Column no. ",cases," containing >",z.warning*100,"% zeros/unobserved values ",action," (can modify threshold using argument z.warning. Check out with zPatterns()).\n",sep=""))
+        }
       }
       
       checkNumZerosRow <- apply(X,1,function(x) sum(is.na(x)))
       if (any(checkNumZerosRow/ncol(X) >= z.warning)) {
         cases <- which(checkNumZerosRow/ncol(X) >= z.warning)
-        X <- X[-cases,]
-        warning(paste("Row ",cases," containing more than ",z.warning*100,"% zeros/unobserved values was deleted (pre-check out using function zPatterns/modify threshold using argument z.warning).\n",sep=""))
+        if (z.delete == TRUE){
+          X <- X[,-cases]
+          action <- "deleted"
+          warning(paste("Column no. ",cases," containing >",z.warning*100,"% zeros/unobserved values ",action," (can modify threshold using argument z.warning).\n",sep=""))
+        }
+        else{
+          action <- "found"
+          stop(paste("Column no. ",cases," containing >",z.warning*100,"% zeros/unobserved values ",action," (can modify threshold using argument z.warning. Check out with zPatterns()).\n",sep=""))
+        }
       }
     }
     
